#include "ParticleSystemCPU.h"

#include <iostream>
using namespace std;


float3 convertToVec(float phi, float theta, float radius)
{
	double PI = 3.14159265;
	float3 ret;
    
	ret.x = float(radius * cos(theta * (PI/180.0))  * cos(phi * (PI/180.0)));
    ret.y = float(radius * cos(theta * (PI/180.0))  * sin(phi * (PI/180.0)));
    ret.z = float(radius * sin(theta * (PI/180.0)));

	return ret;
}



float length(float3 vec)
{
	return (float)sqrt(vec.x*vec.x + vec.y*vec.y + vec.z*vec.z);
}

float3 sub(float3 v1, float3 v2)
{
	float3 ret;
	ret.x = v1.x - v2.x;
	ret.y = v1.y - v2.y;
	ret.z = v1.z - v2.z;

	return ret;
}

float3 add(float3 v1, float3 v2)
{
	float3 ret;
	ret.x = v1.x + v2.x;
	ret.y = v1.y + v2.y;
	ret.z = v1.z + v2.z;

	return ret;
}

float3 mul(float3 v1, double n)
{
	float3 ret;
	ret.x = v1.x * n;
	ret.y = v1.y * n;
	ret.z = v1.z * n;

	return ret;
}

float3 unitVector(float3 vec)
{
	double l = length(vec);
	float3 ret;
	ret.x = (1.0/l) * vec.x;
	ret.y = (1.0/l) * vec.y;
	ret.z = (1.0/l) * vec.z;

	return ret;
}

float3 projectToSphere(float3 vec, float radius)
{
	float3 uVec = unitVector(vec);
	float3 ret = mul(uVec, radius);
	return ret;
}

void computeForces(float3* pos_in, float3* force_out, int n)
{
	for(int i = 0; i < n; ++i)
	{
		force_out[i].x = 0;
		force_out[i].y = 0;
		force_out[i].z = 0;

		for(int j = 0; j < n; ++j)
		{
			if(j == i)
				continue;

			float3 dVec = sub(pos_in[i], pos_in[j]);
			//float3 uVec = unitVector(dVec);

			double l = length(dVec);
			//l = 1.0/(l*10); // ??
			//float3 fVec = mul(uVec, l);

			float3 fVec = mul(dVec, 1.0/(l*l*10));

			force_out[i] = add(force_out[i], fVec);
		}
	}
}

void updatePosition(float3* pos, float3* force_in, int n, float l)
{
	for(int i = 0; i < n; ++i)
	{
		pos[i] = add(pos[i], mul(force_in[i], 1.0/l));
		pos[i] = projectToSphere(pos[i], 1.0f);
	}
}

ParticleSystemCPU::ParticleSystemCPU(int numParticle)
{
	UpMove = 3;
	this->numParticle = numParticle;
	pos = new float3[numParticle];
	force = new float3[numParticle];

	_pos = new float[numParticle*3];
	_force = new float[numParticle*3];

	initialize();
}

void ParticleSystemCPU::initialize()
{
	for(int i=0; i<this->numParticle; ++i)
	{
		pos[i] = convertToVec(rand()%360, rand()%360, 1.0);
		force[i].x = 0;
		force[i].y = 0;
		force[i].z = 0;

		_pos[i*3]	= pos[i].x;
		_pos[i*3+1] = pos[i].y;
		_pos[i*3+2] = pos[i].z;

		_force[i*3]		= 0;
		_force[i*3+1]	= 0;
		_force[i*3+2]	= 0;
	}
}

void ParticleSystemCPU::update()
{
	computeForces(pos, force, numParticle);
	updatePosition(pos, force, numParticle, UpMove);
	UpMove += 0.2;
	cout<<UpMove<<endl;
}

void ParticleSystemCPU::draw()
{
	glBegin(GL_POINTS);
	glColor3d(1.0, 1.0, 1.0);
		for (int i = 0; i < numParticle; ++i)
		{
			glVertex3f(pos[i].x, pos[i].y, pos[i].z);
		}
	glEnd();
}