
#include "template_kernel.cu"
#include <cutil.h>

extern "C"
{

void cudaInit(int argc, char **argv)
{   
    CUT_DEVICE_INIT(argc, argv);
}

void allocateParticleArrays(float* vec, int numParticle)
{
    unsigned int memSize = sizeof( float) * 3 * numParticle;
	cudaMalloc((void**)&vec, memSize);
}

void deleteParticleArrays(float* vec)
{
	cudaFree((void**)vec);
}

void copyArrayFromDevice(float* host, 
                         const float* device, 
                         int numParticle)
{   
	cudaMemcpy(host, device, numParticle * 3 * sizeof(float), cudaMemcpyDeviceToHost);
}

void copyArrayToDevice(float* device, const float* host, int numParticle)
{
	int size = numParticle * 3 * sizeof(float);
	cudaMemcpy(device, host, size, cudaMemcpyHostToDevice);
}

void computeParticle(float* pos,float* force, int numParticle, int p, int q)
{
	int sharedMemSize = p * q * sizeof(float3);
    dim3 threads(p,q,1);
    dim3 grid(numParticle/p, 1, 1);

	computeForces<<< grid, threads, sharedMemSize >>>((float3*)pos, (float3*)force, numParticle);
	updatePosition<<< grid, threads, sharedMemSize >>>((float3*)pos, (float3*)force, numParticle);

	CUT_CHECK_ERROR("Kernel execution failed");
}

}