#ifndef _TEMPLATE_KERNEL_H_
#define _TEMPLATE_KERNEL_H_

#include <stdio.h>

/*
void computeForces(float3* pos_in, float3* force_out, int n)
{
	for(int i = 0; i < n; ++i)
	{
		force_out[i].x = 0;
		force_out[i].y = 0;
		force_out[i].z = 0;

		for(int j = 0; j < n; ++j)
		{
			if(j == i)
				continue;

			float3 dVec = sub(pos_in[i], pos_in[j]);
			//float3 uVec = unitVector(dVec);

			double l = length(dVec);
			//l = 1.0/(l*10); // ??
			//float3 fVec = mul(uVec, l);

			float3 fVec = mul(dVec, 1.0/(l*l*10));

			force_out[i] = add(force_out[i], fVec);
		}
	}
}
*/
__global__ void computeForces(float3* pos_in, float3* force_out, int numParticle)
{
	int index = __mul24(blockIdx.x,blockDim.x) + threadIdx.x;
	float3 pos = pos_in[index];

	force_out[index].x = 0;
	force_out[index].y = 0;
	force_out[index].z = 0;

	for(int i = 0; i < numParticle; ++i)
	{
		if(i == index)
			continue;

		float3 vec;
		vec.x = pos_in[index].x - pos_in[i].x; 
		vec.y = pos_in[index].y - pos_in[i].y; 
		vec.z = pos_in[index].z - pos_in[i].z; 

		float d = sqrtf(vec.x * vec.x + vec.y * vec.y + vec.z * vec.z);

		vec.x *= 1/(d*d*10);
		vec.y *= 1/(d*d*10);
		vec.z *= 1/(d*d*10);

		force_out[index].x += vec.x;
		force_out[index].y += vec.y;
		force_out[index].z += vec.z;

	}
}

/*
void updatePosition(float3* pos, float3* force_in, int n, float l)
{
	for(int i = 0; i < n; ++i)
	{
		pos[i] = add(pos[i], mul(force_in[i], 1.0/l));
		pos[i] = projectToSphere(pos[i], 1.0f);
	}
}
*/

__global__ void updatePosition(float3* position, float3* force_in, int numParticle)
{
	int index = __mul24(blockIdx.x,blockDim.x) + threadIdx.x;
	float3 pos = position[index];
	float3 force = force_in[index];

	force.x *= 1.0f/10.0f;
	force.y *= 1.0f/10.0f;
	force.z *= 1.0f/10.0f;

	pos.x += force.x;
	pos.y += force.y;
	pos.z += force.z;

	// project to Sphere radius 1

	float d = sqrtf(pos.x * pos.x + pos.y * pos.y + pos.z * pos.z);
	// radius = 1
	pos.x *= 1/d;
	pos.y *= 1/d;
	pos.z *= 1/d;

}


#endif // #ifndef _TEMPLATE_KERNEL_H_
